<?php
/**
* Модуль наполнения сайта Mp3GO
*
* @copyright Mp3GO
* @version   1.3.0
* @link      https://mp3go.dev/
*/

date_default_timezone_set('Europe/Moscow');

define('DATALIFEENGINE', true);
define('ROOT_DIR', dirname (__FILE__));
define('ENGINE_DIR', ROOT_DIR.'/engine');

require_once ENGINE_DIR . '/classes/mysql.php';
require_once ENGINE_DIR . '/data/dbconfig.php';
require_once ENGINE_DIR.'/data/config.php';
include_once ROOT_DIR.'/language/'.$config['langs'].'/website.lng';
require_once ENGINE_DIR . '/inc/include/functions.inc.php';

$__config = file_get_contents(ENGINE_DIR . '/data/config.mp3go.php');
$mp3go_config = json_decode($__config, 1);

$added_counter = 0;
$new_tracks = [];
$main_tracks = [];

// Проверка наличия и парсинг новых треков
$__data = send_mp3go_request('tracks.new', $mp3go_config['api_key']);
$new_tracks = $__data['result'];

$new_tracks = array_map(function ($item) {
   $item['is_new_track'] = 1;
   return $item;
}, $new_tracks);

// Парсинг общей коллекции треков
if ($mp3go_config['add_old_track']) {
   $reverse_mode = rand(0, 1);

   if ($reverse_mode)
      $sql = $db->query("SELECT MIN(track_id) FROM ". PREFIX ."_post");
   else
      $sql = $db->query("SELECT MAX(track_id) FROM ". PREFIX ."_post");

   $last_track = $db->get_array($sql);

   if ($last_track[0]) {
      $options_for_request = array('id_start' => $last_track[0], 'reverse' => $reverse_mode ? 1 : '', 'limit' => 10);
      $__data = send_mp3go_request('tracks.last_by_id', $mp3go_config['api_key'], $options_for_request);
      $main_tracks = $__data['result'];
   }
}

// Слияние основных и новых треков
$all_tracks_to_add = array_merge($new_tracks, $main_tracks);

// Публикация собранных треков на сайт
foreach ($all_tracks_to_add as $track) {
   $found_id = $db->query("SELECT id FROM ". PREFIX ."_post WHERE track_id = '{$track['id']}'");

   if ($found_id->num_rows > 0)
      continue;

   if ($mp3go_config['max_length_title'] > 0 && strlen($track['title']) > $mp3go_config['max_length_title'])
      continue;

   // Подготовка данных перед сохранением в БД
   $track_name = "{$track['artist_name']} - {$track['title']}";
   $date = date('Y-m-d H:i:s');
   $letter_index = $config['create_catalog'] && preg_match('#[a-zА-Я]#', $track_name, $results) ? $db->safesql(dle_substr(htmlspecialchars(strip_tags(stripslashes($results[0])), ENT_QUOTES, $config['charset']), 0, 1, $config['charset'])) : '';
   $category = null;
   $cat_ids = [];
   $xf_from_search = [];

   if (count($track['genre_objs'])) {
      foreach ($track['genre_objs'] as $genre) {
         $cat_ids[] = $mp3go_config['cat_genres'][$genre['id']];
         $category = implode(',', array_unique($cat_ids));
      }
   }

   // Если трек является новинкой, добавить в отдельную категорию
   if (isset($track['is_new_track']) && $mp3go_config['cat_genres']['new_tracks'])
      $category .= ',' . $mp3go_config['cat_genres']['new_tracks'];

   $alt_cat_name = totranslit( stripslashes( $track_name ), true, false );
   $xfields_for_db = '';

   // Заполнение дополнительных полей данными
   foreach ($mp3go_config['xfields'] as $field_key => $field_name) {

      // Список доп. полей, для инсерта в таблицу поиска
      $xf_from_search[] = [$field_name, $track[$field_key]];

      // Список доп. полей, для инсерта в БД вместе с постом
      if ($field_key === 'image')
         $xfields_for_db .= $field_name .'|' . upload_poster($track['image']) . '||';
      else if ($field_key === 'genres') {
         $genres = array_map(function ($item) {
            return $item['name'];
         }, $track['genre_objs']);

         $xfields_for_db .= $field_name .'|' . implode(', ', array_unique($genres)) . '||';
      }
      else
         $xfields_for_db .= $field_name .'|' . $track[$field_key] . '||';
   }

   // Обработка доп. полей для перекрестных ссылок
   $xfields = xfieldsload();
   $field_search_on = [];

   // Выборка и заполнение данными полей, для которых разрешены перекрестные ссылки
   $xf_search_words = [];
   foreach ($xf_from_search as $field) {
      foreach ($xfields as $item) {
         if ($field[0] === $item[0] && $item[6]) {
            $xf_search_words[] = $field;
            break;
         }
      }
   }

   $date = $db->safesql($date);
   $xfields_for_db = $db->safesql($xfields_for_db);
   $track_name = $db->safesql($track_name);

   $seo['meta_title'] = str_replace('{track}', "{$track_name}", $mp3go_config['meta_title']);
   $seo['meta_description'] = str_replace('{track}', "{$track_name}", $mp3go_config['meta_description']);
   $seo['meta_title'] = $db->safesql($seo['meta_title']);
   $seo['meta_description'] = $db->safesql($seo['meta_description']);

   // Сохранение трека в БД
   $db->query("INSERT INTO " . PREFIX . "_post (date, autor, short_story, full_story, xfields, title, descr, keywords, category, alt_name, allow_comm, approve, allow_main, fixed, allow_br, symbol, tags, metatitle, track_id) values ('{$date}', '{$mp3go_config['author']}', '', '', '{$xfields_for_db}', '{$track_name}', '{$seo['meta_description']}', '', '{$category}', '{$alt_cat_name}', '1', '1', '1', '0', '1', '{$letter_index}', '', '{$seo['meta_title']}', '{$track['id']}')");
   $row = $db->insert_id();
   $db->query( "INSERT INTO " . PREFIX . "_post_extras (news_id, allow_rate, votes, disable_index, user_id) VALUES('{$row}', '1', '0', '0', '1')" );
   $db->query( "INSERT INTO " . PREFIX . "_tags (news_id, tag) VALUES ('{$row}', '')");

   if ($config['version_id'] + 0 >= 13.2) {

      // Штатные костыли CMS для работы категорий
      $cat_ids = [];
      $cat_ids_arr = explode(',', $category);
      foreach ($cat_ids_arr as $value) {
          $cat_ids[] = "('" . $row . "', '" . trim($value) . "')";
      }

      $cat_ids = implode(', ', $cat_ids);
      $db->query( "INSERT INTO " . PREFIX . "_post_extras_cats (news_id, cat_id) VALUES " . $cat_ids);

      // Штатные костыли CMS для работы перекрестных ссылок
      if (count($xf_search_words)) {
         $temp_array = [];
         foreach ($xf_search_words as $value) {
            $temp_array[] = "('" . $row . "', '" . $value[0] . "', '" . $value[1] . "')";
         }

         $xf_search_words = implode(", ", $temp_array);
         $db->query("INSERT INTO " . PREFIX . "_xfsearch (news_id, tagname, tagvalue) VALUES " . $xf_search_words);
      }
   }

   $added_counter++;
}

print('Добавлено записей: ' . $added_counter);


function send_mp3go_request ($method, $api_key, $options = []) {
   $results = file_get_contents('https://mp3go.dev/api/'. $method .'?' . http_build_query($options) . '&api_key=' . $api_key);
   return json_decode($results, 1);
}

function upload_poster ($url) {
   $dir = date("Y-m");
   $file = file_get_contents($url);
   $file_name = md5($url) . '_' . basename($url);

   $dir_path = ROOT_DIR . '/uploads/posts/' . $dir;
   if (!is_dir($dir_path))
      mkdir($dir_path, 0777, true);

   file_put_contents(ROOT_DIR . '/uploads/posts/' . $dir .'/'. $file_name, $file);
   return "/uploads/posts/{$dir}/{$file_name}";
}